/****************************************************************************************/
/* Led.ino                                                                              */
/* Collection of subroutines to control Neopixel LED WS2812                             */
/*                                                                                      */
/* SUB_FAST_LED_SHOW_ESP32     Subroutine to show LEDs                                  */
/* SUB_FAST_LED_SHOW_TASK      Subroutine to show task                                  */
/* SUB_NEO_SEND_COLOR          Subroutine send Neopixel single color value              */
/****************************************************************************************/


/****************************************************************************************/
/* Sub routine show Leds                                                                */
/* Call this function instead of FastLED.show(). It signals core 0 to issue a show,     */
/* then waits for a notification that it is done.                                       */
/****************************************************************************************/ 
void SUB_FAST_LED_SHOW_ESP32(void)
{
  if (USER_TASK_HANDLE == 0)                      /* User task handler is zero ? */                   
    {
      /* Store the handle of the current task, so that the show task can notify it when it's done */
      USER_TASK_HANDLE = xTaskGetCurrentTaskHandle();
      xTaskNotifyGive(FAST_LED_SHOW_TASK_HANDLE); /* Trigger the show task */
      /* Wait to be notified that it's done */
      const TickType_t xMaxBlockTime = pdMS_TO_TICKS( 200 );
      ulTaskNotifyTake(pdTRUE, xMaxBlockTime);
      USER_TASK_HANDLE = 0;
    }
}


/****************************************************************************************/
/* Sub routine show Task                                                                */
/* This function runs on core 0 and just waits for requests to call FastLED.show()      */
/****************************************************************************************/ 
void SUB_FAST_LED_SHOW_TASK(void *pvParameters)
{
  for(;;)                                         /* Run forever */
  {
    ulTaskNotifyTake(pdTRUE, portMAX_DELAY);      /* Wait for the trigger */
    FastLED.show();                               /* Do the show (synchronously)*/
    xTaskNotifyGive(USER_TASK_HANDLE);            /* Notify the calling task */
  }
}


/****************************************************************************************/
/* Subroutine send Neopixel single color value                                          */
/****************************************************************************************/
void SUB_NEO_SEND_COLOR(byte GREEN_VALUE, byte RED_VALUE, byte BLUE_VALUE)
{
  /* Set color */              
  leds[0] = CRGB( GREEN_VALUE, RED_VALUE, BLUE_VALUE); 
  leds[1] = CRGB( GREEN_VALUE, RED_VALUE, BLUE_VALUE);
  SUB_FAST_LED_SHOW_ESP32();                      /* This sends the updated pixel color to the hardware. */
}
