/****************************************************************************************/
/* Wifi.ino                                                                             */
/* Collection of subroutines for Wifi                                                   */
/* SUB_WLAN_CONNECT:       Subroutine to connect to WIFI                                */          
/****************************************************************************************/


/****************************************************************************************/
/* Subroutine to connect to WIFI                                                        */
/****************************************************************************************/
void SUB_WLAN_CONNECT (void)
{
  uint16_t WLAN_COUNTER;                          /* Wifi counter to check successfull connection */
  int      WLAN_STATUS;                           /* Actual Wifi status */
  boolean  WLAN_CONNECT;                          /* True = Connected to Wifi, False = Not connected to WIFI */  
  
  WLAN_CONNECT = true;                            /* Set Wifi connect = true */
  do                                              /* Repeat until Wifi is connected */
  {
    #ifdef _DEBUG_
      Serial.println("  Starting to connect to WLAN ...");
      Serial.print("    Disconnecting from Wifi: ");  
    # endif 
    /* Step A: Disconnect from WLAN if still connected */
    if (WiFi.status() == WL_CONNECTED)            /* Wifi connected? */
    {
      WiFi.disconnect();                          /* Disconnect from Wifi */
      #ifdef _DEBUG_
        Serial.println("Disconnected");
      # endif 
    }  
    else                                          /* Wifi not connected? */
    {
      #ifdef _DEBUG_
        Serial.println("No connection");
      # endif 
    }   
    
    /* Step B: Set hostname and setting fixed IP-Address */
    #ifdef _DEBUG_
      Serial.print("    Setting fixed IP address: "); 
    # endif
    /* Set fixed IP address failes ? */
    if (!WiFi.config(FIXED_IP, GATEWAY, SUBNET)) 
    {
      #ifdef _DEBUG_
        Serial.println("Failed");
      # endif
      WLAN_CONNECT = false;                       /* Set Wifi connect = true = Error */
    }  
    else                                          /* Set fixed IP address OK ? */
    {
      #ifdef _DEBUG_
        Serial.println("Done");
      # endif
    }

    /* Step C: Disable Wifi Sleep and config Station mode */
    WiFi.setSleep(false);                         /* Switch sleep mode off */ 
    WiFi.mode (WIFI_STA);                         /* Configure station mode */
    
    if (WLAN_CONNECT == true)                     /* No error yet ? */
    {
      /* Step D: Connecting to Wifi Network */
      #ifdef _DEBUG_
        Serial.print("    Connecting to WLAN ");
        Serial.print(WLAN_SSID); 
        Serial.print(": "); 
      # endif
      WiFi.begin(WLAN_SSID, WLAN_PWD);            /* Connecting to WLAN */
      
      WLAN_COUNTER = 0;                           /* Set Wifi counter 0 */
      /* Wifi connected or 20 trials done ? */
      while ((WiFi.status() != WL_CONNECTED) && (WLAN_COUNTER < 20)) 
      {
        delay (100);                              /* Wait hundret miliseconds */
        WLAN_COUNTER++;                           /* Increment Wifi counter */
      }

      if (WLAN_COUNTER >= 20)                     /* Error connecting */
      {
        #ifdef _DEBUG_
          Serial.println("Failed");
        # endif
        WiFi.disconnect();                        /* Disconnect from Wifi */
        delay (1000);                             /* Waiting one second */
        #ifdef _DEBUG_
          Serial.print("    Reconnecting to WLAN ");
          Serial.print(WLAN_SSID); 
          Serial.print(": "); 
        # endif  
        WiFi.begin(WLAN_SSID, WLAN_PWD);          /* Connecting to WLAN */
        WLAN_COUNTER = 0;                         /* Set Wifi counter 0 */
        /* Wifi connected or 20 trials done ? */
        while ((WiFi.status() != WL_CONNECTED) && (WLAN_COUNTER < 20)) 
        {
          delay (100);                            /* Wait hundret miliseconds */
          WLAN_COUNTER++;                         /* Increment Wifi counter */
        }
      }
      
      if (WLAN_COUNTER >= 20)                     /* Error connecting */
      {
        #ifdef _DEBUG_
          Serial.println("Failed");
        # endif  
        WiFi.disconnect();                        /* Wifo disconnect */
        WiFi.mode (WIFI_OFF);                     /* Switch Wifi off */
        SUB_ERROR_ROUTINE ();                     /* Call sub routine for error handling in endless red blinking loop */
      }
      else                                        /* Wifi connected sucessfully ? */
      {
        #ifdef _DEBUG_
          Serial.println("Done");        
          /* Display Wifi information */
          Serial.print("      IP address: ");
          Serial.println(WiFi.localIP());
          Serial.print("      ESP Mac Address: ");
          Serial.println(WiFi.macAddress());
          Serial.print("      Subnet Mask: ");
          Serial.println(WiFi.subnetMask());
          Serial.print("      Gateway IP: ");
          Serial.println(WiFi.gatewayIP());        
          Serial.print("      Signal strength (RSSI): ");
          Serial.print(WiFi.RSSI());
          Serial.println(" dBm");
          Serial.print("      WLAN status: ");
          WLAN_STATUS = WiFi.status();
          switch (WLAN_STATUS)
          {
            case 255:
              Serial.println("WL_NO_SHIELD");
              break;
            case 0:
              Serial.println("WL_IDLE_STATUS");
              break;
            case 1:
              Serial.println("WL_NO_SSID_AVAIL");
              break;
            case 2:
              Serial.println("WL_SCAN_COMPLETED");
              break;
            case 3:
              Serial.println("WL_CONNECTED");
              break;
            case 4:
              Serial.println("WL_CONNECT_FAILED");
              break;
            case 5:
              Serial.println("WL_CONNECTION_LOST");
              break;
            case 6:
              Serial.println("WL_DISCONNECTED");
              break;
            default:
              Serial.println("_UNKNOWN");
              break;
          }                   
        # endif 
        WLAN_CONNECT = true;                      /* Set WLAN connect flag to true */
      }
    }       
  }
  while (WLAN_CONNECT == false);
}
